import 'dart:async';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/view/chats_list_view.dart';
import 'package:flutter/material.dart';
import 'package:rxdart/subjects.dart';
import 'package:socket_io_client/socket_io_client.dart' as io_soc;
import 'package:socket_io_client/socket_io_client.dart';

var params = {"secretKey": AppStorages.secKey, "userId": AppStorages.myUserId};
io_soc.Socket socket = io_soc.io(Global.baseUrl,
    OptionBuilder().setTransports(['websocket']).disableAutoConnect().build());
StreamSocket chatListSoket = StreamSocket();
StreamSocket receivedMessageSocket = StreamSocket();

class StreamSocket {
  StreamController<dynamic> socketResponse = BehaviorSubject();
  void Function(dynamic) get addResponse => socketResponse.sink.add;
  Stream<dynamic> get getResponse => socketResponse.stream;
}

void getGroupList() {
  socket.onConnect((_) {
    socket.emit('get-group-list', params);
  });
  socket.emit('get-group-list', params);
  socket.on('get-group-list-res', (data) {
    chatListSoket.addResponse(data);
  });
  socket.onDisconnect((_) {
    debugPrint('disconnect');
    socket.io
      ..disconnect()
      ..connect();
  });
}

void getConversation(
    {groupId, userId, startAt, Function(dynamic success)? onSuccess}) {
  var convoParam = {
    "groupId": groupId,
    "userId": userId,
    "startAt": startAt,
    "secretKey": AppStorages.secKey,
  };
  socket.connect();
  socket.emit('get-chat', convoParam);
  socket.on('get-chat-res', (data) {
    if (data != null) {
      onSuccess!(data);
    }
  });
}

void sendMessage(param) async {
  socket.emit('send-message', param);
}

void disconnected() async {
  socket.emit('disconnected');
}

void onlineUser() async {
  socket.emit('online', params);
}

void receivedMessages({Function(dynamic success)? success}) {
  socket.connect();
  socket.on('receive-message', (data) {
    if (data != null) {
      success!(data);
    }
  });
  socket.onDisconnect((_) => debugPrint('disconnect'));
}

void typingSocket({groupId, userId, secKey, name, isTyping}) {
  var typeParam = {
    'groupId': groupId,
    'userId': userId,
    "secretKey": secKey,
    "name": name,
    "isTyping": isTyping
  };

  socket.emit('typing', typeParam);
}

void typingResSocket({Function(dynamic onSuccess)? success}) {
  socket.on("typing-res", (data) {
    if (data != null) {
      success!(data);
    }
  });
}

void userActiveStatus({Function(dynamic onSuccess)? success}) {
  socket.on("online-status", (data) {
    if (data != null) {
      success!(data);
    }
  });
  socket.onDisconnect((_) => debugPrint('disconnect'));
}

void clearChat({groupId, userId, Function? onSuccess}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
    "secretKey": AppStorages.secKey
  };
  socket.emit("clear-chat", params);
  socket.on("clear-chat-res", (data) {
    if (data['isSuccess']) {
      if (onSuccess != null) {
        onSuccess();
      }
      // getPreviousMessages(id: userId, grpId: groupId);
    }
  });
}

void leaveConvo({groupId, userId}) async {
  var leaveConvoParam = {
    "groupId": groupId,
    "userId": userId,
    "secretKey": AppStorages.secKey
  };
  socket.emit('leave-group', leaveConvoParam);
}

void getGroup({groupId, Function(dynamic success)? onSuccess}) {
  var groupDetailParam = {
    "groupId": groupId,
    "userId": AppStorages.myUserId,
    "secretKey": AppStorages.secKey
  };
  socket.onConnect((_) {
    socket.emit('get-group', groupDetailParam);
  });
  socket.emit('get-group', groupDetailParam);
  socket.on('get-group-res', (data) {
    if (data != null) {
      onSuccess!(data);
    }
  });
  socket.onDisconnect((_) {
    debugPrint('disconnect');
    socket.io
      ..disconnect()
      ..connect();
  });
}

void updateGroupInfo(param, {context}) {
  socket.emit("update-group", param);
  socket.on("update-group-res", (data) {
    Navigator.pushReplacement(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) => const ChatsListView()));
  });
}

void exitGroup({groupId, userId, context}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
  };
  socket.emit("exit-group", params);
  socket.on("exit-group-res", (data) {
    if (data['isSuccess']) {
      // Get.off(() => const ChatsListView());
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => const ChatsListView()));
    }
  });
}

void deleteGroup({groupId, userId, context}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
  };
  socket.emit("delete-group", params);
  socket.on("delete-group-res", (data) {
    if (data['isSuccess']) {
      // Get.off(() => const ChatsListView());
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => const ChatsListView()));
    }
  });
}
