// ignore_for_file: must_be_immutable

import 'dart:convert';
import 'dart:io';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/keyboard_avoider.dart';
import 'package:aplus_chat/helper/colors.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:aplus_chat/helper/images.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/view/chats_list_view.dart';
import 'package:aplus_chat/view/widgets/comman.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class CreateGroup extends StatefulWidget {
  CreateGroup({Key? key, this.selectedUserIds}) : super(key: key);

  // ignore: prefer_typing_uninitialized_variables
  var selectedUserIds;
  @override
  CreateGroupState createState() => CreateGroupState();
}

class CreateGroupState extends State<CreateGroup> {
  TextEditingController groupNameController = TextEditingController();
  // ignore: prefer_typing_uninitialized_variables
  var imageBytes;
  var groupImage = File("").obs;

  var selectedUserIds = [];
  var imageUploadLoader = false.obs;

  @override
  void initState() {
    super.initState();
    selectedUserIds = [];
    selectedUserIds = widget.selectedUserIds;
    selectedUserIds.add(AppStorages.myUserId);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        backgroundColor: themeColor,
        elevation: 0.8,
        title: Text(
          "Create Group",
          style:
              CustomTextStyles.medium(fontSize: 16.0, fontColor: Colors.black),
        ),
        iconTheme: const IconThemeData(color: Colors.black),
      ),
      body: Column(
        children: [
          const SizedBox(height: 10),
          InkWell(
            borderRadius: BorderRadius.circular(200),
            onTap: () async {
              imageUploadLoader.value = true;
              File pickedFile;
              FilePickerResult? result = await FilePicker.platform.pickFiles(
                type: FileType.media,
                withData: true,
              );
              imageBytes = result!.files[0].bytes;
              pickedFile = File(result.paths[0].toString());
              groupImage.value = pickedFile;
              uploadImageApi(
                  file: imageBytes,
                  fileName:
                      result.paths[0].toString().split("/").last.toString());
            },
            child: Stack(
              alignment: Alignment.center,
              children: [
                Obx(
                  () => !groupImage.value.isAbsolute
                      ? Container(
                          height: 200,
                          width: 200,
                          decoration: const BoxDecoration(
                            color: Colors.black12,
                            shape: BoxShape.circle,
                            image: DecorationImage(
                              image: ExactAssetImage(
                                  ImageResource.groupPlaceHolder),
                              fit: BoxFit.cover,
                            ),
                          ),
                        )
                      : ClipRRect(
                          borderRadius: BorderRadius.circular(200),
                          child: Container(
                            color: Colors.black12,
                            child: Image.file(
                              File(
                                groupImage.value.path,
                              ),
                              height: 200.0,
                              width: 200.0,
                            ),
                          ),
                        ),
                ),
                const Icon(Icons.camera_alt_outlined, color: Colors.white),
              ],
            ),
          ),
          const SizedBox(height: 10),
          Expanded(
            child: KeyboardAvoider(
              autoScroll: true,
              child: Column(
                children: [
                  Padding(
                    padding: const EdgeInsets.only(
                        left: 20, right: 20, top: 8, bottom: 8),
                    child: TextField(
                      controller: groupNameController,
                      onChanged: (val) {},
                      decoration: InputDecoration(
                        hintText: "Enter group name",
                        hintStyle: CustomTextStyles.medium(
                            fontSize: 14.0, fontColor: const Color(0xffBFBFC4)),
                        filled: true,
                        fillColor: const Color(0xffEEEEEF),
                        border: commonBorderStyle,
                        enabledBorder: commonBorderStyle,
                        disabledBorder: commonBorderStyle,
                        focusedBorder: commonBorderStyle,
                        focusedErrorBorder: commonBorderStyle,
                        errorBorder: commonBorderStyle,
                        contentPadding:
                            const EdgeInsets.only(left: 6, right: 6),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          )
        ],
      ),
      bottomNavigationBar: SafeArea(
        child: GestureDetector(
            onTap: () {
              if (groupNameController.text.isEmpty) {
                Common.getSnackToast(context,
                    title: "Error", message: "Please enter group name");
              } else {
                var createGRPParam = {
                  "secretKey": AppStorages.secKey,
                  "userId": AppStorages.myUserId,
                  "email": AppStorages.myEmail.value,
                  "members": selectedUserIds,
                  "isGroup": true,
                  "groupImage": selectedGRPImageUrl,
                  "name": groupNameController.text,
                  "pinGroupForAll": 1,
                  "groupPermission": []
                };
                Webservice.postRequest(
                    context: Get.context,
                    uri: Global.createGroup,
                    body: createGRPParam,
                    baseUrl: Global.baseApiUrl,
                    hasBearer: false,
                    onSuccess: (response) {
                      if (response != null) {
                        Navigator.pushReplacement(
                            context,
                            MaterialPageRoute(
                                builder: (BuildContext context) =>
                                    const ChatsListView()));
                      }
                    },
                    onFailure: (response) {},
                    onConnectionFailed: (response) {});
              }
            },
            child: Obx(
              () => imageUploadLoader.value
                  ? const Center(
                      child: CircularProgressIndicator(),
                    )
                  : Container(
                      height: 50,
                      margin: const EdgeInsets.only(
                          top: 16, left: 20, right: 20, bottom: 10),
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(8.0),
                        color: const Color(0xff0D659D),
                      ),
                      child: Center(
                        child: Text(
                          "Create Group",
                          style: CustomTextStyles.medium(
                              fontSize: 18.0, fontColor: Colors.white),
                        ),
                      ),
                    ),
            )),
      ),
    );
  }

  // Common border for textField
  var commonBorderStyle = const OutlineInputBorder(
    borderSide: BorderSide(color: Color(0xffE6E6E6), width: 1.0),
  );
  var selectedGRPImageUrl = "";
  var selectGRPImageName = "";

  uploadImageApi({contentType, file, fileName, pickdFile, fileType}) async {
    var body = {"contentType": "image/jpg", "file": file, "fileName": fileName};
    Webservice.postRequest(
      uri: Global.uploadFile,
      body: body,
      baseUrl: Global.baseApiUrl,
      hasBearer: false,
      onSuccess: (response) {
        imageUploadLoader.value = false;
        var res = jsonDecode(response);
        selectedGRPImageUrl = res['file'];
        selectGRPImageName = res['fileName'];
      },
      onFailure: (v) {
        imageUploadLoader.value = false;
      },
    );
  }
}
