// ignore_for_file: prefer_typing_uninitialized_variables

import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:ui';
import 'package:aplus_chat/core/services/stream_socket.dart';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:aplus_chat/helper/images.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/helper/utilites.dart';
import 'package:aplus_chat/model/conversation_list_model.dart';
import 'package:aplus_chat/view/contact_info_view.dart';
import 'package:aplus_chat/view/widgets/comman.dart';
import 'package:aplus_chat/view/widgets/group_info_view.dart';
import 'package:aplus_chat/view/widgets/image_view_dialog.dart';
import 'package:aplus_chat/view/widgets/video_player.dart';
import 'package:better_open_file/better_open_file.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:grouped_list/grouped_list.dart';
import 'package:image_picker/image_picker.dart';
import 'package:scroll_to_index/scroll_to_index.dart';
import 'package:swipe_to/swipe_to.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:video_thumbnail/video_thumbnail.dart';
import 'widgets/audio_play_dialog.dart';

// ignore: must_be_immutable
class MessageView extends StatefulWidget {
  MessageView({Key? key, this.groupId, this.isGroup, this.isOnlineCall
      //this.opponentUserId,
      })
      : super(key: key);

  String? groupId;
  bool? isGroup;
  bool? isOnlineCall = false;
  // String? opponentUserId;
  @override
  MessageViewState createState() => MessageViewState();
}

class MessageViewState extends State<MessageView>
    with SingleTickerProviderStateMixin {
  var info;
  RxList<Message> conversationList = <Message>[].obs;
  var hasMore = false;
  var startType = true;
  GroupData? groupData;
  var otherUserTyping = false.obs;
  var otherTyingUserName = "".obs;
  var isUserOnline = false.obs;
  var hasActionPermission = true.obs;
  var hasSendMessagePermission = true;
  RxList<PopupMenuEntry<int>> actionList = <PopupMenuEntry<int>>[].obs;
  late Animation<Color?> animation;
  late AnimationController controller;
  // GroupListModel convoData = GroupListModel();
  var groupId = "";
  var groupName = "", groupImage = "";
  var opponentUserId = "";

  @override
  void initState() {
    if (widget.isOnlineCall!) {
      onlineUser();
    }
    // convoData = widget.groupListData!;
    groupId = widget.groupId!;
    typingResSocket(success: (val) {
      otherUserTyping.value = val['isTyping'] == "true";
      otherTyingUserName.value = val['name'];
    });
    userActiveStatus(success: (val) {
      isUserOnline.value = val['isOnline'];
    });
    getConversation(
        groupId: groupId,
        userId: AppStorages.myUserId,
        startAt: 0,
        onSuccess: (val) {
          ConversationListModel conversationListModel =
              conversationListModelFromJson(val);

          conversationList.addAll(conversationListModel.messages!.reversed);
          hasMore = conversationListModel.hasMore!;

          groupData = conversationListModel.groupData!;
          if (groupData != null) {
            groupName = groupData!.groupName!;
            groupImage = groupData!.imagePath!;
            opponentUserId = groupData!.opponentUserId!;
          }
          if (groupData != null &&
              groupData!.userPermission!.permission != null) {
            isUserOnline.value = groupData!.onlineStatus!;

            if (widget.isGroup!) {
              hasSendMessagePermission =
                  groupData!.userPermission!.permission!.sendMessage == 1;
              if (groupData!.userPermission!.permission!.clearChat == 1) {
                actionList.add(PopupMenuItem(
                  onTap: () {
                    clearChat(
                        userId: AppStorages.myUserId,
                        groupId: groupId,
                        onSuccess: () {
                          conversationList.clear();
                        });
                  },
                  value: 3,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Text(
                        "Clear chat",
                        style: CustomTextStyles.medium(
                            fontSize: 16.0, fontColor: Colors.black),
                      ),
                      const Icon(Icons.message_outlined),
                    ],
                  ),
                ));
              }
              if (groupData!.userPermission!.permission!.deleteChat == 1) {
                actionList.add(PopupMenuItem(
                  value: 2,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Text(
                        "Delete chat",
                        style: CustomTextStyles.medium(
                            fontSize: 16.0, fontColor: Colors.black),
                      ),
                      const Icon(Icons.delete),
                    ],
                  ),
                  onTap: () {
                    deleteGroup(
                        userId: AppStorages.myUserId,
                        groupId: groupId,
                        context: context);
                  },
                ));
              }
            } else {
              actionList.add(PopupMenuItem(
                value: 2,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Text(
                      "Delete chat",
                      style: CustomTextStyles.medium(
                          fontSize: 16.0, fontColor: Colors.black),
                    ),
                    const Icon(Icons.delete),
                  ],
                ),
                onTap: () {
                  deleteGroup(
                      userId: AppStorages.myUserId,
                      groupId: groupId,
                      context: context);
                },
              ));
              actionList.add(PopupMenuItem(
                onTap: () {
                  clearChat(
                      userId: AppStorages.myUserId,
                      groupId: groupId,
                      onSuccess: () {
                        conversationList.clear();
                      });
                },
                value: 3,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Text(
                      "Clear chat",
                      style: CustomTextStyles.medium(
                          fontSize: 16.0, fontColor: Colors.black),
                    ),
                    const Icon(Icons.message_outlined),
                  ],
                ),
              ));
            }
          }

          receivedMessages(success: (data) {
            Message messages = messagesFromJson(data);
            if (messages.sentBy != AppStorages.myUserId) {
              conversationList.insert(0, messages);
            }
          });
        });

    autoScrollController = AutoScrollController(keepScrollOffset: true);
    autoScrollController.addListener(scroll);

    /// BLINK ANIMATION ////

    controller = AnimationController(
      duration: const Duration(milliseconds: 600),
      reverseDuration: const Duration(seconds: 1),
      vsync: this,
    );
    final CurvedAnimation curve =
        CurvedAnimation(parent: controller, curve: Curves.linear);
    animation = ColorTween(
            begin: const Color(0xff8EB4C9).withOpacity(0.8), end: Colors.blue)
        .animate(curve);
    // Keep the animation going forever once it is started
    animation.addStatusListener((status) {
      // Reverse the animation after it has been completed
      if (status == AnimationStatus.completed) {
        controller.reverse();
      }
    });
    super.initState();
  }

  @override
  dispose() {
    controller.dispose();
    super.dispose();
  }

  scroll() {
    if (autoScrollController.position.maxScrollExtent ==
        autoScrollController.position.pixels) {
      if (hasMore) {
        var convoParam = {
          "groupId": groupId,
          "userId": AppStorages.myUserId,
          "startAt": conversationList[conversationList.length - 1].time! - 1,
          "secretKey": AppStorages.secKey,
        };
        socket.emit('get-chat', convoParam);
      }
    } else {
      if (autoScrollController.position.pixels == 0) {
        debugPrint("TOP");
      }
    }
  }

  AutoScrollController autoScrollController = AutoScrollController();
  willPop() {
    if (widget.isOnlineCall!) {
      disconnected();
    }
    leaveConvo(groupId: groupId, userId: AppStorages.myUserId);
    Navigator.pop(context);
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: () => willPop(),
        child: Scaffold(
          body: SafeArea(
            top: false,
            bottom: true,
            child: Container(
              decoration: const BoxDecoration(
                color: Colors.white,
                image: DecorationImage(
                  image: ExactAssetImage(ImageResource.chatBg,
                      package: "aplus_chat"),
                  fit: BoxFit.cover,
                ),
              ),
              child: Column(
                children: [
                  appBar(),
                  Expanded(
                    child: Obx(
                      () => GroupedListView<dynamic, String>(
                        // ignore: invalid_use_of_protected_member
                        elements: conversationList.value,
                        groupBy: (element) => Utilities.convertTimeStampToDate(
                            element.timeMilliSeconds.seconds,
                            formate: 'yyyy-MM-dd'),
                        order: GroupedListOrder.DESC,
                        reverse: true,
                        controller: autoScrollController,
                        stickyHeaderBackgroundColor: Colors.transparent,
                        floatingHeader: true,
                        separator: const SizedBox(height: 0),
                        useStickyGroupSeparators: true,
                        groupSeparatorBuilder: (value) {
                          return groupSapeatorItem(value);
                        },
                        indexedItemBuilder: (context, item, index) {
                          return AutoScrollTag(
                              key: ValueKey(index),
                              controller: autoScrollController,
                              index: index,
                              child: messageItem(index, conversationList[index],
                                  animation: animation));
                        },
                      ),
                    ),
                  ),
                  messageSendView(),
                ],
              ),
            ),
          ),
        ));
  }

  groupSapeatorItem(value) {
    String date = ChatDateFormatter().groupChatDateHeader(value);
    return SizedBox(
      height: 40,
      child: Align(
        alignment: Alignment.center,
        child: Container(
          padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 10),
          decoration: const BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(5.0)),
          ),
          child: Text(
            date,
            style: CustomTextStyles.normal(
                fontSize: 10.0, fontColor: Colors.black),
            textAlign: TextAlign.center,
          ),
        ),
      ),
    );
  }

  Widget appBar() {
    return SafeArea(
      child: Container(
        color: const Color(0xffE5E5EA),
        padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 0),
        child: Row(
          children: [
            IconButton(
                onPressed: () {
                  leaveConvo(groupId: groupId, userId: AppStorages.myUserId);
                  socket.off("get-chat-res");
                  Navigator.pop(context);
                },
                icon: const Icon(Icons.arrow_back_outlined)),
            GestureDetector(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => !widget.isGroup!
                          ? ContactInfoView(userId: opponentUserId)
                          : GroupInfoView(groupId: groupId),
                    ));
              },
              child: Common.profileNetworkImage(
                borderRadius: 60.0,
                height: 40.0,
                width: 40.0,
                placeHolder: widget.isGroup!
                    ? ImageResource.groupPlaceHolder
                    : ImageResource.profilePlaceHolder,
                imageUrl: groupImage,
              ),
            ),
            Obx(
              () => Expanded(
                child: GestureDetector(
                  onTap: () {
                    debugPrint(opponentUserId);
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => !widget.isGroup!
                            ? ContactInfoView(userId: opponentUserId)
                            : GroupInfoView(groupId: groupId),
                      ),
                    );
                  },
                  child: isUserOnline.value || otherUserTyping.value
                      ? Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            Text(groupName,
                                style: CustomTextStyles.semiBold(
                                    fontSize: 18.0, fontColor: Colors.black)),
                            otherUserTyping.value
                                ? Text(
                                    otherUserTyping.value
                                        ? widget.isGroup!
                                            ? "${otherTyingUserName.value} Typing ..."
                                            : " Typing..."
                                        : "",
                                    style: CustomTextStyles.medium(
                                        fontSize: 14.0, fontColor: Colors.blue),
                                  )
                                : isUserOnline.value
                                    ? Text(
                                        " Online",
                                        style: CustomTextStyles.medium(
                                            fontSize: 14.0,
                                            fontColor: Colors.green),
                                      )
                                    : const SizedBox.shrink(),
                          ],
                        )
                      : Text(groupName,
                          style: CustomTextStyles.semiBold(
                              fontSize: 18.0, fontColor: Colors.black)),
                ),
              ),
            ),
            Obx(() => !hasActionPermission.value
                ? const SizedBox.shrink()
                : PopupMenuButton<int>(
                    icon: const Icon(
                      Icons.more_vert_outlined,
                      color: Colors.black,
                    ),
                    // ignore: invalid_use_of_protected_member
                    itemBuilder: (context) => actionList.value,
                    offset: const Offset(0, 50),
                    color: Colors.white70,
                    elevation: 2,
                  )),
          ],
        ),
      ),
    );
  }

  var isReplied = false.obs;
  Rx<Message> repliedMessage = Message().obs;
  messageItem(index, Message message, {animation}) {
    final CurvedAnimation curve =
        CurvedAnimation(parent: controller, curve: Curves.linear);
    animation = message.sentBy == AppStorages.myUserId
        ? ColorTween(
                begin: const Color(0xff8EB4C9).withOpacity(0.8),
                end: Colors.white)
            .animate(curve)
        : ColorTween(
                begin: Colors.white,
                end: const Color(0xff8EB4C9).withOpacity(0.8))
            .animate(curve);
    return AnimatedBuilder(
        animation: animation,
        builder: (BuildContext context, Widget? child) {
          return SwipeTo(
            onRightSwipe: message.sentBy == AppStorages.myUserId
                ? null
                : () {
                    repliedMessage.value = message;
                    isReplied.value = true;
                    FocusScope.of(context).requestFocus(focusNode);
                  },
            onLeftSwipe: message.sentBy == AppStorages.myUserId
                ? () {
                    repliedMessage.value = message;
                    isReplied.value = true;
                    FocusScope.of(context).requestFocus(focusNode);
                  }
                : null,
            child: InkWell(
              onLongPress: () {},
              child: Padding(
                padding: EdgeInsets.only(
                  left: message.sentBy == AppStorages.myUserId ? 100.0 : 10,
                  right: message.sentBy == AppStorages.myUserId ? 10 : 80,
                  top: 10,
                  bottom: 10,
                ),
                child: Row(
                  mainAxisSize: MainAxisSize.min,
                  mainAxisAlignment: message.sentBy == AppStorages.myUserId
                      ? MainAxisAlignment.end
                      : MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.end,
                  children: [
                    const SizedBox(width: 6),
                    Expanded(
                      child: Column(
                        crossAxisAlignment:
                            message.sentBy == AppStorages.myUserId
                                ? CrossAxisAlignment.end
                                : CrossAxisAlignment.start,
                        children: [
                          Container(
                            padding: const EdgeInsets.all(8),
                            decoration: BoxDecoration(
                              color: message.startAnimation.value
                                  ? animation.value
                                  : message.sentBy == AppStorages.myUserId
                                      ? const Color(0xff8EB4C9).withOpacity(0.8)
                                      : Colors.white,
                              borderRadius: BorderRadius.circular(6),
                            ),
                            child: Padding(
                              padding: const EdgeInsets.all(0.0),
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  widget.isGroup!
                                      ? message.sentBy != AppStorages.myUserId
                                          ? Padding(
                                              padding: const EdgeInsets.only(
                                                  bottom: 8),
                                              child: Text(
                                                message.senderName!,
                                                style: CustomTextStyles.bold(
                                                    fontSize: 14.0,
                                                    fontColor: Colors.blue),
                                              ),
                                            )
                                          : const SizedBox.shrink()
                                      : const SizedBox.shrink(),
                                  messageType(message, index),
                                ],
                              ),
                            ),
                          ),
                          const SizedBox(height: 8),
                          Text(
                            Utilities.convertTimeStampToTime(
                                message.timeMilliSeconds!.seconds),
                            textAlign: message.sentBy == AppStorages.myUserId
                                ? TextAlign.right
                                : TextAlign.left,
                            style: CustomTextStyles.normal(
                                fontSize: 8.0, fontColor: Colors.black),
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
          );
        });
  }

  messageType(Message chat, index) {
    var fileName;
    if (chat.type == "text") {
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          chat.replyMsgId == ""
              ? const SizedBox.shrink()
              : InkWell(
                  onTap: () async {
                    for (var element in conversationList) {
                      if (element.startAnimation.value) {
                        element.startAnimation.value = false;
                      }
                    }
                    var selcIndex = conversationList
                        .indexWhere((item) => item.msgId == chat.replyMsgId);

                    if (selcIndex != -1) {
                      await autoScrollController.scrollToIndex(selcIndex,
                          preferPosition: AutoScrollPosition.end);
                      conversationList[selcIndex].startAnimation.value = true;
                      controller.forward();
                    }
                  },
                  child: Container(
                    margin: const EdgeInsets.only(right: 20, bottom: 8),
                    decoration: BoxDecoration(
                      color: Colors.grey[200],
                      borderRadius: BorderRadius.circular(8),
                    ),
                    child: IntrinsicHeight(
                      child: Row(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Container(
                            width: 4.0,
                            decoration: const BoxDecoration(
                              color: Colors.deepPurpleAccent,
                              borderRadius: BorderRadius.only(
                                topLeft: Radius.circular(8.0),
                                bottomLeft: Radius.circular(8.0),
                              ),
                            ),
                          ),
                          Expanded(
                              child: Padding(
                            padding: const EdgeInsets.only(
                                top: 12, bottom: 12, left: 12),
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Text(
                                  chat.replyUserId == AppStorages.myUserId
                                      ? "You"
                                      : chat.replyUser!,
                                  style: CustomTextStyles.medium(
                                      fontSize: 14.0,
                                      fontColor: Colors.deepPurple),
                                  overflow: TextOverflow.ellipsis,
                                ),
                                const SizedBox(height: 4),
                                Text(
                                  chat.replyMsgType != "text"
                                      ? chat.replyMsgType!
                                      : chat.replyMsg!,
                                  style: CustomTextStyles.normal(
                                      fontSize: 12.0, fontColor: Colors.black),
                                  overflow: TextOverflow.ellipsis,
                                )
                              ],
                            ),
                          )),
                          chat.replyMsgType != "text"
                              ? ClipRRect(
                                  borderRadius: BorderRadius.circular(4.0),
                                  child: chat.replyMsg!.isURL
                                      ? Hero(
                                          tag: 'htg_0',
                                          child: CachedNetworkImage(
                                            fadeInDuration: const Duration(
                                                milliseconds: 250),
                                            height: 60,
                                            width: 52,
                                            imageUrl: chat.replyMsg!,
                                            errorWidget:
                                                (context, url, error) =>
                                                    Opacity(
                                              opacity: 0.9,
                                              child: ClipRRect(
                                                borderRadius:
                                                    BorderRadius.circular(30.0),
                                                child: Image.asset(
                                                  ImageResource
                                                      .groupPlaceHolder,
                                                  package: "aplus_chat",
                                                ),
                                              ),
                                            ),
                                            fit: BoxFit.cover,
                                            placeholder: ((context, String s) =>
                                                Image.asset(
                                                  ImageResource
                                                      .groupPlaceHolder,
                                                  height: 40,
                                                  width: 40,
                                                  package: "aplus_chat",
                                                )),
                                          ))
                                      : Image.file(
                                          File(
                                            chat.replyMsg!,
                                          ),
                                          height: 60,
                                          width: 52,
                                        ))
                              : const SizedBox.shrink(),
                        ],
                      ),
                    ),
                  ),
                ),
          Text(chat.message!,
              style: CustomTextStyles.medium(
                  fontSize: 12.0, fontColor: Colors.black),
              textAlign: chat.sentBy == AppStorages.myUserId
                  ? TextAlign.right
                  : TextAlign.left),
        ],
      );
    } else if (chat.type == "audio") {
      return InkWell(
        onTap: () async {
          Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => Player(
                      audioFile: chat.filePath, fileName: chat.fileName)));
        },
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              mainAxisSize: MainAxisSize.min,
              children: [
                Flexible(
                  fit: FlexFit.loose,
                  child: Column(
                    children: [
                      Text(
                        'Audio File',
                        style: CustomTextStyles.medium(
                            fontSize: 12.0,
                            fontColor: Colors.black,
                            isUnderLine: true),
                      ),
                      if (fileName != null)
                        Text(
                          '$fileName',
                          style: CustomTextStyles.normal(
                              fontSize: 8.0, fontColor: Colors.black),
                        ),
                    ],
                  ),
                ),
                const SizedBox(width: 5),
                Container(
                  height: 35,
                  width: 35,
                  decoration: BoxDecoration(
                    color: Colors.grey,
                    borderRadius: BorderRadius.circular(40),
                  ),
                  child: const Icon(
                    Icons.audiotrack,
                    color: Colors.white,
                    size: 20,
                  ),
                ),
              ],
            ),
          ],
        ),
      );
    } else if (chat.type == "document") {
      return InkWell(
        onTap: () async {
          if (chat.filePath!.isURL) {
            if (!await launchUrl(
              Uri.parse(chat.filePath!),
              mode: LaunchMode.externalApplication,
            )) {
              throw 'Could not launch ${chat.filePath}';
            }
          } else {
            debugPrint("OPEN FILE");
            OpenFile.open(chat.filePath);
          }
        },
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              mainAxisSize: MainAxisSize.min,
              children: [
                Flexible(
                  fit: FlexFit.loose,
                  child: Column(
                    children: [
                      Text(
                        'Document File',
                        style: CustomTextStyles.medium(
                            fontSize: 12.0,
                            fontColor: Colors.black,
                            isUnderLine: true),
                      ),
                      if (fileName != null)
                        Text(
                          '$fileName',
                          style: CustomTextStyles.normal(
                              fontSize: 8.0, fontColor: Colors.black),
                        ),
                    ],
                  ),
                ),
                const SizedBox(width: 5),
                Container(
                  height: 35,
                  width: 35,
                  decoration: BoxDecoration(
                    color: Colors.grey,
                    borderRadius: BorderRadius.circular(40),
                  ),
                  child: const Icon(
                    Icons.attach_file,
                    color: Colors.white,
                    size: 20,
                  ),
                ),
              ],
            ),
          ],
        ),
      );
    } else if (chat.type == "image") {
      debugPrint(chat.filePath);
      return InkWell(
        onTap: () {
          showDialog(
              context: context,
              barrierDismissible: false,
              builder: (BuildContext context) {
                return ImagePreview(image: chat.filePath);
              });
        },
        child: ClipRRect(
          borderRadius: BorderRadius.circular(10),
          child: Container(
              height: 200,
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10),
              ),
              child: chat.filePath!.isURL
                  ? Hero(
                      tag: 'htg_$index',
                      child: CachedNetworkImage(
                        fadeInDuration: const Duration(milliseconds: 250),
                        imageUrl: chat.filePath!,
                        width: 140,
                        errorWidget: (context, url, error) => Opacity(
                          opacity: 0.9,
                          child: ClipRRect(
                            borderRadius: BorderRadius.circular(30.0),
                            child: Image.asset(
                              ImageResource.groupPlaceHolder,
                              package: "aplus_chat",
                            ),
                          ),
                        ),
                        fit: BoxFit.cover,
                        placeholder: ((context, String s) => Image.asset(
                              ImageResource.groupPlaceHolder,
                              // height: 40,
                              // width: 40,
                              package: "aplus_chat",
                            )),
                      ))
                  : Stack(alignment: Alignment.center, children: [
                      Image.file(File(chat.filePath!)),
                      chat.fileUploadLoader.value
                          ? const SizedBox(
                              // height: 40,
                              // width: 40,
                              child: CircularProgressIndicator())
                          : const SizedBox.shrink(),
                    ])),
        ),
      );
    } else if (chat.type == "video") {
      return InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => VideoPlayerView(
                videoFile: chat.filePath,
                isVideoUrl: chat.filePath!.isURL,
              ),
            ),
          );
        },
        child: ClipRRect(
          borderRadius: BorderRadius.circular(10),
          child: Stack(
            alignment: AlignmentDirectional.center,
            children: [
              Container(
                height: 200,
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                ),
                child: chat.thumbnailPath!.isURL
                    ? Hero(
                        tag: 'htg_$index',
                        child: CachedNetworkImage(
                          fadeInDuration: const Duration(milliseconds: 250),
                          imageUrl: chat.thumbnailPath!,
                          errorWidget: (context, url, error) => Opacity(
                            opacity: 0.9,
                            child: ClipRRect(
                              borderRadius: BorderRadius.circular(30.0),
                              child: Image.asset(
                                ImageResource.groupPlaceHolder,
                                package: "aplus_chat",
                              ),
                            ),
                          ),
                          fit: BoxFit.cover,
                          placeholder: ((context, String s) => Image.asset(
                                ImageResource.groupPlaceHolder,
                                height: 40,
                                width: 40,
                                package: "aplus_chat",
                              )),
                        ),
                      )
                    : Image.file(
                        File(chat.thumbnailPath!),
                      ),
              ),
              const Icon(Icons.play_circle)
            ],
          ),
        ),
      );
    }
  }

  TextEditingController messageSendController = TextEditingController();
  var sendButtonEnable = false.obs;
  var isTyping = false.obs;
  var focusNode = FocusNode();
  messageSendView() {
    return Container(
      padding: const EdgeInsets.only(top: 6, bottom: 6),
      decoration: const BoxDecoration(color: Color(0xffD1D1D5)),
      child: Column(
        children: [
          Obx(() => isReplied.value
              ? Container(
                  margin: const EdgeInsets.only(left: 20, right: 20, bottom: 8),
                  decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius: BorderRadius.circular(8),
                  ),
                  child: IntrinsicHeight(
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Container(
                          width: 4.0,
                          decoration: const BoxDecoration(
                            color: Colors.deepPurpleAccent,
                            borderRadius: BorderRadius.only(
                              topLeft: Radius.circular(8.0),
                              bottomLeft: Radius.circular(8.0),
                            ),
                          ),
                        ),
                        Expanded(
                            child: Padding(
                          padding: const EdgeInsets.only(
                              top: 12, bottom: 12, left: 12),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                repliedMessage.value.sentBy ==
                                        AppStorages.myUserId
                                    ? "You"
                                    : repliedMessage.value.senderName!,
                                style: CustomTextStyles.medium(
                                    fontSize: 14.0,
                                    fontColor: Colors.deepPurple),
                                overflow: TextOverflow.ellipsis,
                              ),
                              const SizedBox(height: 4),
                              Text(
                                repliedMessage.value.type == "text"
                                    ? repliedMessage.value.message!
                                    : repliedMessage.value.type!,
                                style: CustomTextStyles.normal(
                                    fontSize: 12.0, fontColor: Colors.black),
                                overflow: TextOverflow.ellipsis,
                              )
                            ],
                          ),
                        )),
                        repliedMessage.value.type != "text"
                            ? Stack(
                                alignment: Alignment.topRight,
                                children: [
                                  ClipRRect(
                                      borderRadius: BorderRadius.circular(4.0),
                                      child: repliedMessage
                                              .value.filePath!.isURL
                                          ? Hero(
                                              tag: 'htg_0',
                                              child: CachedNetworkImage(
                                                fadeInDuration: const Duration(
                                                    milliseconds: 250),
                                                height: 60,
                                                width: 52,
                                                imageUrl: repliedMessage
                                                    .value.filePath!,
                                                errorWidget:
                                                    (context, url, error) =>
                                                        Opacity(
                                                  opacity: 0.9,
                                                  child: ClipRRect(
                                                    borderRadius:
                                                        BorderRadius.circular(
                                                            30.0),
                                                    child: Image.asset(
                                                      ImageResource
                                                          .groupPlaceHolder,
                                                      package: "aplus_chat",
                                                    ),
                                                  ),
                                                ),
                                                fit: BoxFit.cover,
                                                placeholder:
                                                    ((context, String s) =>
                                                        Image.asset(
                                                          ImageResource
                                                              .groupPlaceHolder,
                                                          height: 40,
                                                          width: 40,
                                                          package: "aplus_chat",
                                                        )),
                                              ))
                                          : Image.file(
                                              File(
                                                repliedMessage.value.filePath!,
                                              ),
                                              height: 60,
                                              width: 52,
                                            )),
                                  InkWell(
                                    onTap: () {
                                      isReplied.value = false;
                                    },
                                    child: Container(
                                      margin: const EdgeInsets.only(
                                          top: 4, right: 4),
                                      decoration: const BoxDecoration(
                                          shape: BoxShape.circle,
                                          color: Colors.white60),
                                      child: const Icon(
                                        Icons.clear,
                                        size: 12,
                                      ),
                                    ),
                                  ),
                                ],
                              )
                            : IconButton(
                                onPressed: () {
                                  isReplied.value = false;
                                },
                                icon: const Icon(Icons.clear),
                              ),
                      ],
                    ),
                  ),
                )
              : const SizedBox.shrink()),
          Row(
            children: [
              IconButton(
                  onPressed: () {
                    showAttachDialog();
                  },
                  icon: const Icon(Icons.attach_file)),
              Expanded(
                  child: TextField(
                // readOnly: true,
                controller: messageSendController,
                onChanged: (text) async {
                  if (!isTyping.value) {
                    startType = true;
                    isTyping.value = true;
                    typingSocket(
                        groupId: groupId,
                        userId: AppStorages.myUserId,
                        secKey: AppStorages.secKey,
                        name: groupData!.userName,
                        isTyping: true);
                    Future.delayed(const Duration(seconds: 2)).whenComplete(() {
                      startType = false;

                      isTyping.value = false;
                      typingSocket(
                          groupId: groupId,
                          userId: AppStorages.myUserId,
                          secKey: AppStorages.secKey,
                          name: groupData!.userName,
                          isTyping: false);
                    });
                  }

                  if (text.isEmpty) {
                    sendButtonEnable.value = false;
                  } else {
                    sendButtonEnable.value = true;
                  }
                },

                maxLines: 7,
                minLines: 1,
                style: const TextStyle(
                  // fontFamily: Styles.FontNameRegular,
                  color: Colors.black,
                ),
                focusNode: focusNode,
                textInputAction: TextInputAction.newline,
                decoration: InputDecoration(
                  fillColor: Colors.white,
                  filled: true,
                  contentPadding: const EdgeInsets.only(left: 10, right: 10),
                  hintText: "Type a message",
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(6),
                    borderSide: BorderSide.none,
                  ),
                ),
              )),
              hasSendMessagePermission
                  ? Obx(
                      () => IconButton(
                        onPressed: sendButtonEnable.value
                            ? () {
                                var sendMsgParam = {
                                  "messageObj": {
                                    "message": messageSendController.text,
                                    "type": "text",
                                    "sentBy": AppStorages.myUserId,
                                    "senderName": groupData!.userName,
                                    "replyUser": isReplied.value
                                        ? repliedMessage.value.senderName
                                        : "",
                                    "replyMsg": isReplied.value
                                        ? repliedMessage.value.message
                                        : "",
                                    "replyMsgId": isReplied.value
                                        ? repliedMessage.value.msgId
                                        : "",
                                    "replyUserId": isReplied.value
                                        ? repliedMessage.value.sentBy
                                        : "",
                                    "replyMsgType":
                                        isReplied.value ? "text" : ""
                                  },
                                  "groupId": groupId,
                                  "secretKey": AppStorages.secKey,
                                  "userId": AppStorages.myUserId
                                };
                                DateTime now = DateTime.now();
                                int timestamp = now.millisecondsSinceEpoch;
                                conversationList.insert(
                                    0,
                                    Message(
                                      time: timestamp ~/ 1000,
                                      timeMilliSeconds: TimeMilliSeconds(
                                          seconds: timestamp ~/ 1000,
                                          nanoseconds: timestamp ~/ 1000),
                                      sentBy: AppStorages.myUserId,
                                      senderName: groupData!.userName,
                                      type: "text",
                                      msgId: "",
                                      message: messageSendController.text,
                                      contentType: "text",
                                      replyUser: isReplied.value
                                          ? repliedMessage.value.senderName
                                          : "",
                                      replyMsg: isReplied.value
                                          ? repliedMessage.value.message
                                          : "",
                                      replyMsgId: isReplied.value
                                          ? repliedMessage.value.msgId
                                          : "",
                                      replyUserId: isReplied.value
                                          ? repliedMessage.value.sentBy
                                          : "",
                                      replyMsgType:
                                          isReplied.value ? "text" : "",
                                      fileName: "",
                                      filePath: "",
                                      name: "",
                                    ));

                                sendMessage(sendMsgParam);
                                autoScrollController.animateTo(
                                  autoScrollController.position.minScrollExtent,
                                  duration: const Duration(milliseconds: 1600),
                                  curve: Curves.easeOut,
                                );
                                isReplied.value = false;
                                messageSendController.clear();
                                sendButtonEnable.value = false;
                              }
                            : null,
                        icon: Icon(
                          Icons.send_sharp,
                          color: sendButtonEnable.value
                              ? Colors.black
                              : Colors.black12,
                        ),
                      ),
                    )
                  : const SizedBox.shrink()
            ],
          ),
        ],
      ),
    );
  }

  void showAttachDialog() {
    var alert = Align(
      alignment: Alignment.bottomCenter,
      child: BackdropFilter(
        filter: ImageFilter.blur(sigmaX: 3, sigmaY: 3),
        child: Padding(
          padding: const EdgeInsets.all(8),
          child: Container(
            width: Get.width / 1.2,
            margin: const EdgeInsets.only(bottom: 30, left: 12, right: 12),
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(10),
              color: Colors.white,
            ),
            child: Material(
              color: Colors.transparent,
              borderRadius: BorderRadius.circular(10),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Align(
                    alignment: Alignment.topRight,
                    child: IconButton(
                      icon: const Icon(Icons.clear),
                      onPressed: () => Navigator.pop(context),
                    ),
                  ),
                  dialogTile(
                      text: 'Send Attachment',
                      fontWeight: FontWeight.bold,
                      fontSize: 18),
                  saperator(),
                  dialogTile(
                      text: 'Gallery',
                      callback: () {
                        pickImage(true);
                      }),
                  saperator(),
                  dialogTile(
                      text: 'Camera',
                      callback: () {
                        pickImage(false);
                      }),
                  saperator(),
                  dialogTile(
                      text: 'Document',
                      callback: () {
                        attachDocument();
                      }),
                ],
              ),
            ),
          ),
        ),
      ),
    );

    showGeneralDialog(
      barrierLabel: "Label",
      barrierDismissible: true,
      barrierColor: Colors.black.withOpacity(0.5),
      transitionDuration: const Duration(milliseconds: 400),
      context: context,
      pageBuilder: (context, anim1, anim2) {
        return alert;
      },
      transitionBuilder: (context, anim1, anim2, child) {
        return SlideTransition(
          position: Tween(begin: const Offset(0, 1), end: const Offset(0, 0))
              .animate(anim1),
          child: child,
        );
      },
    );
  }

  dialogTile(
      {required String text,
      Function? callback,
      fontWeight = FontWeight.normal,
      double fontSize = 16}) {
    return InkWell(
      onTap: () {
        if (callback != null) {
          Navigator.pop(context);
          callback();
        }
      },
      child: Container(
        width: Get.width,
        alignment: Alignment.center,
        child: Padding(
          padding: const EdgeInsets.only(
              right: 12.0, left: 12.0, top: 16, bottom: 16),
          child: Text(
            text,
            style: TextStyle(
              fontSize: fontSize,
              fontWeight: fontWeight,
            ),
          ),
        ),
      ),
    );
  }

  saperator() {
    return Padding(
        padding: const EdgeInsets.only(left: 24, right: 24),
        child: Container(height: 1, color: Colors.grey));
  }

  String? thumbnails = "";
  Future pickImage(gallery) async {
    // String thumbnail = "";
    ImagePicker picker = ImagePicker();
    File pickedFile;

    if (gallery) {
      FilePickerResult? result = await FilePicker.platform.pickFiles(
        type: FileType.media,
        withData: true,
      );

      var bytes = result!.files[0].bytes;
      var imageName = result.files[0].name;
      pickedFile = File(result.paths[0].toString());
      debugPrint(pickedFile.path.split("/").last);
      DateTime now = DateTime.now();
      int timestamp = now.millisecondsSinceEpoch;

      if (result.files.first.extension == "mp4" ||
          result.files.first.extension == "mov" ||
          result.files.first.extension == "mkv") {
        thumbnails = await VideoThumbnail.thumbnailFile(
            video: result.paths[0].toString(), imageFormat: ImageFormat.JPEG);
      }
      conversationList.insert(
          0,
          Message(
            time: timestamp ~/ 1000,
            timeMilliSeconds: TimeMilliSeconds(
                seconds: timestamp ~/ 1000, nanoseconds: timestamp ~/ 1000),
            sentBy: AppStorages.myUserId,
            senderName: groupData!.userName,
            type: result.names.toString().split(".").last.contains("mp4")
                ? "video"
                : "image",
            msgId: "",
            message: "",
            contentType: "file",
            replyMsg: "",
            replyMsgId: "",
            replyMsgType: "",
            replyUser: "",
            replyUserId: "",
            fileName: pickedFile.path.split("/").last,
            filePath: pickedFile.path,
            name: "",
          ));
      conversationList[0].fileUploadLoader.value = true;
      uploadImageApi(
        fileType: result.names.toString().split(".").last.contains("mp4")
            ? "video"
            : "image",
        fileName: imageName,
        file: bytes,
        contentType: result.names.toString().split(".").last.contains("mp4")
            ? "video/${result.files.first.extension}"
            : "image/${result.files.first.extension}",
        pickdFile: pickedFile,
      );
    }

    // Otherwise open camera to get new photo
    else {
      final XFile? image = await picker.pickImage(
        source: ImageSource.camera,
        imageQuality: 80,
      );

      List<int> bytes = await File(image!.path).readAsBytes();
      var imageName = image.name;
      pickedFile = File(image.path.toString());

      DateTime now = DateTime.now();
      int timestamp = now.millisecondsSinceEpoch;
      conversationList.insert(
          0,
          Message(
            time: timestamp ~/ 1000,
            timeMilliSeconds: TimeMilliSeconds(
                seconds: timestamp ~/ 1000, nanoseconds: timestamp ~/ 1000),
            sentBy: AppStorages.myUserId,
            senderName: groupData!.userName,
            type: "image",
            msgId: "",
            message: "",
            contentType: "image",
            replyMsg: "",
            replyMsgId: "",
            replyMsgType: "",
            replyUser: "",
            replyUserId: "",
            fileName: pickedFile.path.split("/").last,
            filePath: pickedFile.path,
            name: "",
          ));
      conversationList[0].fileUploadLoader.value = true;
      uploadImageApi(
        fileType: "image",
        fileName: imageName,
        file: bytes,
        contentType: "image/jpg",
        pickdFile: pickedFile,
      );
    }
  }

  File? file;

  void attachDocument() async {
    File pickedFile;
    FilePickerResult? result = await FilePicker.platform.pickFiles(
      type: FileType.custom,
      allowedExtensions: ['txt', 'pdf', 'doc', 'xlsx', 'mp3'],
    );

    if (result != null) {
      var bytes = result.files[0].bytes;
      var imageName = result.files[0].name;
      pickedFile = File(result.paths[0].toString());
      DateTime now = DateTime.now();
      int timestamp = now.millisecondsSinceEpoch;
      conversationList.insert(
          0,
          Message(
            time: timestamp ~/ 1000,
            timeMilliSeconds: TimeMilliSeconds(
                seconds: timestamp ~/ 1000, nanoseconds: timestamp ~/ 1000),
            sentBy: AppStorages.myUserId,
            senderName: groupData!.userName,
            type: "document",
            msgId: "",
            message: "",
            contentType: "file",
            replyMsg: "",
            replyMsgId: "",
            replyMsgType: "",
            replyUser: "",
            replyUserId: "",
            fileName: pickedFile.path.split("/").last,
            filePath: pickedFile.path,
            name: "",
          ));
      conversationList[0].fileUploadLoader.value = true;
      uploadImageApi(
        fileType: "document",
        fileName: imageName,
        file: bytes,
        contentType: "document/${result.files.first.extension}",
        pickdFile: pickedFile,
      );
    } else {
      debugPrint('User canceled the picker');
    }
  }

  uploadImageApi(
      {contentType, file, fileName, pickdFile, fileType, type}) async {
    var body = {
      "file": file,
      "fileName": fileName,
      "contentType": contentType,
      "secretKey": AppStorages.secKey,
      "type": fileType
    };

    Webservice.postRequest(
      uri: Global.uploadFile,
      body: body,
      baseUrl: Global.baseApiUrl,
      hasBearer: false,
      onSuccess: (response) {
        debugPrint(response);
        var res = jsonDecode(response);
        var sendMsgParam = {
          "messageObj": {
            "message": "",
            "type": fileType,
            "sentBy": AppStorages.myUserId,
            "senderName": groupData!.userName,
            "fileName": res['fileName'],
            "contentType": contentType,
            "filePath": pickdFile.path,
            "replyUser": "",
            "replyMsg": "",
            "replyMsgId": "",
            "replyUserId": "",
            "replyMsgType": "",
            "file": res['file'],
          },
          "groupId": groupId,
          "secretKey": AppStorages.secKey,
          "userId": AppStorages.myUserId
        };
        conversationList[0].fileUploadLoader.value = false;
        conversationList.refresh();
        sendMessage(sendMsgParam);
        autoScrollController.animateTo(
          autoScrollController.position.minScrollExtent,
          duration: const Duration(milliseconds: 1600),
          curve: Curves.easeOut,
        );
        sendButtonEnable.value = false;
      },
      onFailure: (v) {
        conversationList[0].fileUploadLoader.value = false;
        conversationList.refresh();
      },
    );
  }
}
