import 'package:aplus_chat/helper/colors.dart';
import 'package:aplus_chat/helper/custom_styles.dart';
import 'package:better_player/better_player.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class VideoPlayerView extends StatefulWidget {
  const VideoPlayerView({
    Key? key,
    this.videoFile,
    this.isVideoUrl,
  }) : super(key: key);
  final String? videoFile;
  final bool? isVideoUrl;
  @override
  VideoPlayerViewState createState() => VideoPlayerViewState();
}

class VideoPlayerViewState extends State<VideoPlayerView> {
  BetterPlayerController? _betterPlayerController;
  BetterPlayerDataSource? _dataSource;
  var mediaUrl = "".obs;

  @override
  void initState() {
    super.initState();
    mediaUrl.value = widget.videoFile!;
    var isUrl = widget.isVideoUrl;
    playVideo(mediaUrl.value, isUrl);
  }

  playVideo(url, isUrl) {
    if (isUrl) {
      _dataSource =
          BetterPlayerDataSource(BetterPlayerDataSourceType.network, url);
    } else {
      _dataSource =
          BetterPlayerDataSource(BetterPlayerDataSourceType.file, url);
    }

    _betterPlayerController = BetterPlayerController(
      BetterPlayerConfiguration(
        expandToFill: false,
        autoPlay: true,
        autoDispose: false,
        fullScreenByDefault: false,
        fit: BoxFit.fill,
        autoDetectFullscreenDeviceOrientation: true,
        errorBuilder: (context, err) {
          return Text(err!);
        },
        controlsConfiguration: const BetterPlayerControlsConfiguration(
          controlBarColor: Colors.transparent,
          iconsColor: Colors.white,
          playIcon: Icons.play_circle_fill_rounded,
          pauseIcon: Icons.pause_circle_filled_outlined,
          progressBarPlayedColor: Colors.grey,
          progressBarHandleColor: Colors.grey,
          enableSkips: true,
          enableFullscreen: true,
          controlBarHeight: 60,
          loadingColor: Colors.white,
          overflowModalColor: Colors.transparent,
          overflowModalTextColor: Colors.white,
          overflowMenuIconsColor: Colors.white,
        ),
      ),
      betterPlayerDataSource: _dataSource,
    );
  }

  @override
  void dispose() {
    super.dispose();
    if (mediaUrl.isNotEmpty) {
      _betterPlayerController!.videoPlayerController!.pause();
      _betterPlayerController!.videoPlayerController!.dispose();
      _betterPlayerController!.dispose();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.black,
      appBar: AppBar(
        backgroundColor: themeColor,
        elevation: 0.8,
        title: Text(
          "Video",
          style: CustomTextStyles.medium(
            fontSize: 16.0,
            fontColor: Colors.black,
          ),
        ),
        iconTheme: const IconThemeData(color: Colors.black),
      ),
      body: Center(
        child: BetterPlayer(
          controller: _betterPlayerController!,
        ),
      ),
    );
  }
}
